from django.urls import reverse
from rest_framework import status

from arkindex.dataimport.models import WorkerVersion
from arkindex.documents.models import Corpus
from arkindex.project.tests import FixtureAPITestCase
from arkindex.users.models import User


class TestTranscriptions(FixtureAPITestCase):
    """
    Tests for text element view
    """

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.page = cls.corpus.elements.get(name='Volume 1, page 1r')
        cls.volume = cls.corpus.elements.get(name='Volume 1')
        cls.line = cls.corpus.elements.get(name='Text line')
        cls.private_corpus = Corpus.objects.create(name='Private')
        cls.private_page = cls.private_corpus.elements.create(type=cls.page.type)
        # Create an user with a read right only on the private corpus
        cls.private_read_user = User.objects.create_user('a@bc.de', 'a')
        cls.private_read_user.verified_email = True
        cls.private_read_user.save()
        cls.worker_version_1 = WorkerVersion.objects.get(worker__slug='reco')
        cls.worker_version_2 = WorkerVersion.objects.get(worker__slug='dla')

    def test_list_transcriptions_read_right(self):
        # A read right on the element corpus is required to access transcriptions
        self.client.force_login(self.private_read_user)
        url = reverse('api:element-transcriptions', kwargs={'pk': str(self.private_page.id)})
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_list_element_transcriptions(self):
        tr1 = self.page.transcriptions.get()
        tr2 = self.page.transcriptions.create(
            text='something',
            worker_version=self.worker_version_2,
            confidence=0.369,
        )
        self.client.force_login(self.user)

        with self.assertNumQueries(11):
            response = self.client.get(reverse('api:element-transcriptions', kwargs={'pk': str(self.page.id)}))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertCountEqual(response.json()['results'], [
            {
                'id': str(tr1.id),
                'text': 'Lorem ipsum dolor sit amet',
                'confidence': 1.0,
                'score': 1.0,
                'worker_version_id': str(self.worker_version_1.id),
                'element': None,
            },
            {
                'id': str(tr2.id),
                'text': 'something',
                'confidence': 0.369,
                'score': 0.369,
                'worker_version_id': str(self.worker_version_2.id),
                'element': None,
            }
        ])

    def test_list_transcriptions_recursive(self):
        self.client.force_login(self.user)

        with self.assertNumQueries(13):
            response = self.client.get(
                reverse('api:element-transcriptions', kwargs={'pk': str(self.page.id)}),
                data={'recursive': 'true'}
            )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertCountEqual(
            [
                (data['element']['type'], data['element']['zone']['polygon'], data['text'])
                for data in response.json()['results']
            ], [
                ('page', [[0, 0], [0, 1000], [1000, 1000], [1000, 0], [0, 0]], 'Lorem ipsum dolor sit amet'),
                ('word', [[100, 100], [100, 200], [200, 200], [200, 100], [100, 100]], 'PARIS'),
                ('word', [[400, 400], [400, 500], [500, 500], [500, 400], [400, 400]], 'ROY'),
                ('word', [[700, 700], [700, 800], [800, 800], [800, 700], [700, 700]], 'DATUM'),
            ]
        )

    def test_list_transcriptions_recursive_filter_element_type(self):
        self.client.force_login(self.user)

        with self.assertNumQueries(11):
            response = self.client.get(
                reverse('api:element-transcriptions', kwargs={'pk': str(self.page.id)}),
                data={'recursive': 'true', 'element_type': 'page'}
            )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertCountEqual(
            [
                (data['element']['type'], data['element']['zone']['polygon'], data['text'])
                for data in response.json()['results']
            ], [
                ('page', [[0, 0], [0, 1000], [1000, 1000], [1000, 0], [0, 0]], 'Lorem ipsum dolor sit amet'),
            ]
        )

    def test_list_worker_version_transcriptions(self):
        worker_transcription = self.page.transcriptions.create(
            text='something',
            worker_version=self.worker_version_2,
            confidence=0.369,
        )

        self.client.force_login(self.user)

        with self.assertNumQueries(13):
            response = self.client.get(
                reverse('api:element-transcriptions', kwargs={'pk': str(self.page.id)}),
                data={'recursive': 'true', 'worker_version': str(self.worker_version_2.id)}
            )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertListEqual(response.json()['results'], [
            {
                'id': str(worker_transcription.id),
                'text': 'something',
                'score': 0.369,
                'confidence': 0.369,
                'worker_version_id': str(self.worker_version_2.id),
                'element': {
                    'id': str(self.page.id),
                    'name': 'Volume 1, page 1r',
                    'type': 'page',
                    'zone': {
                        'id': str(self.page.id),
                        'polygon': [[0, 0], [0, 1000], [1000, 1000], [1000, 0], [0, 0]],
                    },
                    'rotation_angle': 0,
                    'mirrored': False
                },
            }
        ])

    def test_list_transcriptions_worker_version_validation(self):
        self.client.force_login(self.user)
        with self.assertNumQueries(6):
            response = self.client.get(
                reverse('api:element-transcriptions', kwargs={'pk': str(self.page.id)}),
                data={'recursive': 'true', 'worker_version': 'oh no'}
            )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json(), {'worker_version': ['“oh no” is not a valid UUID.']})
